@@ -1,35 +1,10 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 Ford Motor Company
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtRemoteObjects module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2017 Ford Motor Company
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#include "modeltest.h"
 #include "../shared/model_utilities.h"
 
 #include <QtTest/QtTest>
+#include <QAbstractItemModelTester>
 #include <QMetaType>
 #include <QRemoteObjectReplica>
 #include <QRemoteObjectNode>
@@ -45,9 +20,9 @@ QList<QStandardItem*> createInsertionChildren(int num, const QString& name, cons
 {
     QList<QStandardItem*> children;
     for (int i = 0; i < num; ++i) {
-        QScopedPointer<QStandardItem> item(new QStandardItem(QStringLiteral("%1 %2").arg(name).arg(i+1)));
+        QStandardItem *item = new QStandardItem(QStringLiteral("%1 %2").arg(name).arg(i+1));
         item->setBackground(background);
-        children.append(item.take());
+        children.append(item);
     }
     return children;
 }
@@ -87,24 +62,22 @@ public:
 
     void scheduleRowsToWatch(const QModelIndex &index, int start, int end)
     {
-        m_pendingRows.push_back(RowData(index, start, end));
+        m_pendingRows.push_back(RowData { index, start, end });
     }
 
-    QVector<QModelIndex> changedData() const { return m_changedData; }
+    QList<QModelIndex> changedData() const { return m_changedData; }
 
 private:
     struct RowData
     {
-        RowData(const QModelIndex &idx = QModelIndex(), int s = -1, int e = -1)
-            : m_index(idx), m_start(s), m_end(e) {}
         QModelIndex m_index;
-        int m_start;
-        int m_end;
+        int m_start = -1;
+        int m_end = -1;
     };
 
     const QAbstractItemModel *m_model;
-    QVector<RowData> m_pendingRows;
-    QVector<QModelIndex> m_changedData;
+    QList<RowData> m_pendingRows;
+    QList<QModelIndex> m_changedData;
     int m_currentRowsCount = 0;
     const int m_expectedRowsCount;
 };
@@ -173,7 +146,7 @@ void compareData(const QAbstractItemModel *sourceModel, const QAbstractItemModel
 }
 
 void compareIndex(const QModelIndex &sourceIndex, const QModelIndex &replicaIndex,
-                  const QVector<int> &roles)
+                  const QList<int> &roles)
 {
     QVERIFY(sourceIndex.isValid());
     QVERIFY(replicaIndex.isValid());
@@ -219,7 +192,7 @@ void compareTreeData(const QAbstractItemModel *sourceModel, const QAbstractItemM
     }
 }
 
-void compareTreeData(const QAbstractItemModel *sourceModel, const QAbstractItemModel *replica, const QVector<int> &roles)
+void compareTreeData(const QAbstractItemModel *sourceModel, const QAbstractItemModel *replica, const QList<int> &roles)
 {
     QVERIFY(sourceModel);
     QVERIFY(replica);
@@ -257,8 +230,8 @@ void compareFlags(const QAbstractItemModel *sourceModel, const QAbstractItemMode
 class RolenamesListModel : public QAbstractListModel
 {
 public:
-    explicit RolenamesListModel(QObject *parent = 0) : QAbstractListModel(parent) { }
-    int rowCount(const QModelIndex &) const override{return m_list.length(); }
+    explicit RolenamesListModel(QObject *parent = nullptr) : QAbstractListModel(parent) { }
+    int rowCount(const QModelIndex &) const override { return int(m_list.size()); }
     QVariant data(const QModelIndex &index, int role) const override
     {
        if (role == Qt::UserRole)
@@ -282,7 +255,7 @@ public:
         m_list.clear();
     }
 private:
-    QVector<QPair<QVariant, QVariant> > m_list;
+    QList<QPair<QVariant, QVariant>> m_list;
 };
 
 QList<QStandardItem*> addChild(int numChilds, int nestingLevel)
@@ -335,17 +308,17 @@ public:
 
 private:
     const QAbstractItemModelReplica *m_replica;
-    QHash<QPersistentModelIndex, QVector<int>> m_pending;
+    QHash<QPersistentModelIndex, QList<int>> m_pending;
     QSet<QPersistentModelIndex> m_waitForInsertion;
 
-    void addData(const QModelIndex &index, const QVector<int> &roles)
+    void addData(const QModelIndex &index, const QList<int> &roles)
     {
         for (int role : roles) {
             const bool cached = m_replica->hasData(index, role);
             if (cached)
                 continue;
             if (!m_pending.contains(index))
-                m_pending[index] = QVector<int>() << role;
+                m_pending[index] = QList<int>() << role;
             else {
                 if (!m_pending[index].contains(role))
                     m_pending[index].append(role);
@@ -353,7 +326,7 @@ private:
         }
     }
 
-    void addIndex(const QModelIndex &parent, const QVector<int> &roles)
+    void addIndex(const QModelIndex &parent, const QList<int> &roles)
     {
         if (parent.isValid())
             addData(parent, roles);
@@ -377,20 +350,20 @@ private:
             finish();
             return;
         }
-        QHash<QPersistentModelIndex, QVector<int> > pending(m_pending);
+        QHash<QPersistentModelIndex, QList<int>> pending(m_pending);
         pending.detach();
-        QHash<QPersistentModelIndex, QVector<int> >::ConstIterator it(pending.constBegin()), end(pending.constEnd());
+        auto it(pending.constBegin()), end(pending.constEnd());
         for (; it != end; ++it) {
             for (int role : it.value()) {
                 QVariant v = m_replica->data(it.key(), role);
-                Q_UNUSED(v);
+                Q_UNUSED(v)
             }
         }
     }
 
     void rowsInserted(const QModelIndex &parent, int first, int last)
     {
-        static QVector<int> rolesV;
+        static QList<int> rolesV;
         if (rolesV.isEmpty())
             rolesV << Qt::DisplayRole << Qt::BackgroundRole;
         m_waitForInsertion.remove(parent);
@@ -415,7 +388,7 @@ private:
             fetch();
     }
 
-    void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles = QVector<int>())
+    void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QList<int> &roles = {})
     {
         Q_ASSERT(topLeft.isValid());
         Q_ASSERT(bottomRight.isValid());
@@ -426,12 +399,12 @@ private:
             for (int j = topLeft.column(); j <= bottomRight.column(); ++j) {
                 const QModelIndex index = m_replica->index(i, j, parent);
                 Q_ASSERT(index.isValid());
-                QHash<QPersistentModelIndex, QVector<int> >::Iterator it = m_pending.find(index);
+                auto it = m_pending.find(index);
                 if (it == m_pending.end())
                     continue;
 
 #if 0
-                QVector<int> itroles = it.value();
+                QList<int> itroles = it.value();
                 itroles.detach();
                 if (roles.isEmpty()) {
                     itroles.clear();
@@ -446,7 +419,7 @@ private:
                     m_pending[index] = itroles;
                 }
 #else
-                Q_UNUSED(roles);
+                Q_UNUSED(roles)
                 m_pending.erase(it);
                 if (m_replica->hasChildren(index)) {
                     // ask for the row count to get an update
@@ -513,6 +486,7 @@ private slots:
 
     void testRoleNames();
 
+    void testModelTest_data();
     void testModelTest();
     void testSortFilterModel();
 
@@ -570,9 +544,9 @@ void TestModelView::setup_models(QRemoteObjectHost &basicServer, QRemoteObjectNo
     static const QString url = QStringLiteral("tcp://127.0.0.1:%1");
 
     // QStandardItem::flags are stored as data with Qt::UserRole - 1
-    static const QVector<int> sourceModelRoles( {Qt::DisplayRole, Qt::BackgroundRole, (Qt::UserRole - 1)} );
+    static const QList<int> sourceModelRoles( {Qt::DisplayRole, Qt::BackgroundRole, (Qt::UserRole - 1)} );
 
-    static const QVector<int> listModelRoles( {Qt::UserRole, Qt::UserRole+1} );
+    static const QList<int> listModelRoles( {Qt::UserRole, Qt::UserRole+1} );
 
     //Setup registry
     //Registry needs to be created first until we get the retry mechanism implemented
@@ -594,7 +568,7 @@ void TestModelView::setup_models(QRemoteObjectHost &basicServer, QRemoteObjectNo
 void TestModelView::testEmptyModel()
 {
     _SETUP_TEST_
-    QVector<int> roles = QVector<int>() << Qt::DisplayRole << Qt::BackgroundRole;
+    QList<int> roles = QList<int>() << Qt::DisplayRole << Qt::BackgroundRole;
     QStandardItemModel emptyModel;
     basicServer.enableRemoting(&emptyModel, "emptyModel", roles);
 
@@ -759,7 +733,7 @@ void TestModelView::testDataChanged()
     FetchData f(model.data());
     QVERIFY(f.fetchAndWait(MODELTEST_WAIT_TIME));
 
-    QSignalSpy dataChangedSpy(model.data(), SIGNAL(dataChanged(QModelIndex,QModelIndex,QVector<int>)));
+    QSignalSpy dataChangedSpy(model.data(),  &QAbstractItemModelReplica::dataChanged);
     for (int i = 10; i < 20; ++i)
         m_sourceModel.setData(m_sourceModel.index(i, 1), QColor(Qt::blue), Qt::BackgroundRole);
 
@@ -783,7 +757,7 @@ void TestModelView::testDataInsertion()
 
     const int insertedRowsCount = 9;
     RowsWatcher watcher(model.data(), insertedRowsCount);
-    QVector<QModelIndex> pending;
+    QList<QModelIndex> pending;
 
     m_sourceModel.insertRows(2, insertedRowsCount);
 
@@ -815,7 +789,7 @@ void TestModelView::testDataInsertionTree()
     const int insertedChildRowsCount = 4;
     RowsWatcher watcher(model.data(), insertedRowsCount + insertedChildRowsCount);
 
-    QVector<QModelIndex> pending;
+    QList<QModelIndex> pending;
 
     for (int i = 0; i < insertedRowsCount; ++i) {
         watcher.scheduleRowsToWatch(QModelIndex(), 2 + i, 2 + i);
@@ -884,7 +858,7 @@ void TestModelView::testDataRemoval()
 
     // change one row to check for inconsistencies
 
-    QVector<QModelIndex> pending;
+    QList<QModelIndex> pending;
     pending << m_sourceModel.index(0, 0, parent);
     WaitForDataChanged w(model.data(), pending);
     m_sourceModel.setData(m_sourceModel.index(0, 0, parent), QColor(Qt::green), Qt::BackgroundRole);
@@ -921,7 +895,7 @@ void TestModelView::testDataRemovalTree()
 void TestModelView::testServerInsertDataTree()
 {
     _SETUP_TEST_
-    QVector<int> roles = QVector<int>() << Qt::DisplayRole << Qt::BackgroundRole;
+    QList<int> roles = QList<int> { Qt::DisplayRole, Qt::BackgroundRole };
     QStandardItemModel testTreeModel;
     basicServer.enableRemoting(&testTreeModel, "testTreeModel", roles);
 
@@ -948,15 +922,29 @@ void TestModelView::testServerInsertDataTree()
     compareData(&testTreeModel, model.data());
 }
 
+void TestModelView::testModelTest_data()
+{
+    // QtRemoteObjects::InitialAction is a namespace enum.  QTest/QFETCH fail to compile with it as
+    // the column type, so use bool instead.
+    QTest::addColumn<bool>("prefetch");
+
+    QTest::newRow("size only") << false;
+    QTest::newRow("prefetch") << true;
+}
+
 void TestModelView::testModelTest()
 {
+    QFETCH(bool, prefetch);
     _SETUP_TEST_
-    QScopedPointer<QAbstractItemModelReplica> repModel( client.acquireModel(QStringLiteral("test")));
-    ModelTest test(repModel.data());
+    QtRemoteObjects::InitialAction action = QtRemoteObjects::InitialAction::FetchRootSize;
+    if (prefetch)
+        action = QtRemoteObjects::InitialAction::PrefetchData;
+    QScopedPointer<QAbstractItemModelReplica> repModel( client.acquireModel(QStringLiteral("test"), action));
+    QAbstractItemModelTester test(repModel.data(), QAbstractItemModelTester::FailureReportingMode::Fatal);
 
     FetchData f(repModel.data());
     QVERIFY(f.fetchAndWait(MODELTEST_WAIT_TIME));
-    Q_UNUSED(test);
+    Q_UNUSED(test)
 }
 
 void TestModelView::testSortFilterModel()
@@ -987,8 +975,8 @@ void TestModelView::testSetData()
     compareTreeData(&m_sourceModel, model.data(), model->availableRoles());
 
     //fetched and verified initial state, now setData on the client
-    QVector<QModelIndex> pending;
-    QVector<QModelIndex> pendingReplica;
+    QList<QModelIndex> pending;
+    QList<QModelIndex> pendingReplica;
     for (int row = 0, numRows = model->rowCount(); row < numRows; ++row) {
         for (int column = 0, numColumns = model->columnCount(); column != numColumns; ++column) {
             const QModelIndex index = model->index(row, column);
@@ -1015,12 +1003,12 @@ void TestModelView::testSetDataTree()
     compareTreeData(&m_sourceModel, model.data(), model->availableRoles());
 
     //fetched and verified initial state, now setData on the client
-    QVector<QModelIndex> pending;
-    QVector<QModelIndex> pendingReplica;
+    QList<QModelIndex> pending;
+    QList<QModelIndex> pendingReplica;
 
-    QVector<QModelIndex> stack;
+    QList<QModelIndex> stack;
     stack.push_back(QModelIndex());
-    QVector<QModelIndex> sourceStack;
+    QList<QModelIndex> sourceStack;
     sourceStack.push_back(QModelIndex());
 
 
@@ -1052,7 +1040,7 @@ void TestModelView::testSetDataTree()
 void TestModelView::testSelectionFromReplica()
 {
     _SETUP_TEST_
-    QVector<int> roles = QVector<int>() << Qt::DisplayRole << Qt::BackgroundRole;
+    QList<int> roles = QList<int> { Qt::DisplayRole, Qt::BackgroundRole };
     QStandardItemModel simpleModel;
     for (int i = 0; i < 4; ++i)
         simpleModel.appendRow(new QStandardItem(QString("item %0").arg(i)));
@@ -1072,7 +1060,7 @@ void TestModelView::testSelectionFromReplica()
 void TestModelView::testSelectionFromSource()
 {
     _SETUP_TEST_
-    QVector<int> roles = QVector<int>() << Qt::DisplayRole << Qt::BackgroundRole;
+    QList<int> roles = QList<int> { Qt::DisplayRole, Qt::BackgroundRole };
     QStandardItemModel simpleModel;
     for (int i = 0; i < 4; ++i)
         simpleModel.appendRow(new QStandardItem(QString("item %0").arg(i)));
@@ -1091,15 +1079,15 @@ void TestModelView::testSelectionFromSource()
 
 void TestModelView::testCacheData_data()
 {
-    QTest::addColumn<QVector<int>>("roles");
+    QTest::addColumn<QList<int>>("roles");
 
-    QTest::newRow("empty") << QVector<int> {};
-    QTest::newRow("all") << QVector<int> { Qt::UserRole, Qt::UserRole + 1 };
+    QTest::newRow("empty") << QList<int> {};
+    QTest::newRow("all") << QList<int> { Qt::UserRole, Qt::UserRole + 1 };
 }
 
 void TestModelView::testCacheData()
 {
-    QFETCH(QVector<int>, roles);
+    QFETCH(QList<int>, roles);
 
     _SETUP_TEST_
     QScopedPointer<QAbstractItemModelReplica> model(client.acquireModel("testRoleNames", QtRemoteObjects::PrefetchData, roles));
@@ -1115,7 +1103,7 @@ void TestModelView::testCacheData()
 void TestModelView::testChildSelection()
 {
     _SETUP_TEST_
-    QVector<int> roles = {Qt::DisplayRole, Qt::BackgroundRole};
+    QList<int> roles = { Qt::DisplayRole, Qt::BackgroundRole };
     QStandardItemModel simpleModel;
     QStandardItem *parentItem = simpleModel.invisibleRootItem();
     for (int i = 0; i < 4; ++i) {
